==> avodel-web-driver-5.2.1-527a4a.zip: src/Mouse/Mouse.php <==
<?php

declare(strict_types=1);

namespace Avodel\WebDriver\Mouse;

use Facebook\WebDriver\Chrome\ChromeDevToolsDriver;
use Facebook\WebDriver\Remote\RemoteWebDriver;

class Mouse
{
    private Coordinate $mouseCoordinate;
    private Coordinate $scrollCoordinate;

    public function __construct(
        private readonly RemoteWebDriver $webDriver,
        private readonly MousePathStrategyInterface $mousePathStrategy,
        /**
         * @var array{scroll: array{min: int, max: int}, move: array{min: int, max: int}}
         */
        private readonly array $delays,
    ) {
        $this->mouseCoordinate = new Coordinate(random_int(600, 900), random_int(300, 600));
        $this->scrollCoordinate = new Coordinate(0, 0);
    }

    public function click(Coordinate $coordinate): void
    {
        $devTools = new ChromeDevToolsDriver($this->webDriver);

        $devTools->execute('Input.dispatchMouseEvent', [
            'type' => 'mousePressed',
            'x' => $coordinate->getX(),
            'y' => $coordinate->getY(),
            'button' => 'left',
            'clickCount' => 1,
        ]);
        $devTools->execute('Input.dispatchMouseEvent', [
            'type' => 'mouseReleased',
            'x' => $coordinate->getX(),
            'y' => $coordinate->getY(),
            'button' => 'left',
            'clickCount' => 1,
        ]);
    }

    /**
     * @return void
     */
    public function move(Coordinate $targetCoordinate): void
    {
        $mousePath = $this->mousePathStrategy->calculatePath(
            $this->mouseCoordinate,
            $targetCoordinate
        );

        $devTools = new ChromeDevToolsDriver($this->webDriver);

        foreach ($mousePath as $coordinate) {
            $devTools->execute('Input.dispatchMouseEvent', [
                'type' => 'mouseMoved',
                'x' => $coordinate->getX(),
                'y' => $coordinate->getY(),
            ]);

            usleep(random_int($this->delays['move']['min'], $this->delays['move']['max']) * 1000);

            $this->mouseCoordinate = $coordinate;
        }
    }

    public function scroll(Coordinate $coordinate): void
    {
        $scrollData = $this->webDriver->executeScript('return {
        scrollTop: window.scrollY,
        scrollHeight: document.documentElement.scrollHeight,
        windowHeight: window.innerHeight
    };');

        $currentScrollPosition = $scrollData['scrollTop'];
        $documentHeight = $scrollData['scrollHeight'];
        $windowHeight = $scrollData['windowHeight'];

        $targetY = $coordinate->getY();
        $deltaY = $targetY - $currentScrollPosition;

        if ($deltaY < 0 && $currentScrollPosition + $deltaY < 0) {
            $deltaY = -$currentScrollPosition;
        }

        if ($currentScrollPosition + $deltaY + $windowHeight >= $documentHeight) {
            $deltaY = $documentHeight - ($currentScrollPosition + $windowHeight);
        }

        $steps = 20;
        $deltaPerStep = $deltaY / $steps;

        $devTools = new ChromeDevToolsDriver($this->webDriver);

        for ($i = 0; $i < $steps; $i++) {
            $devTools->execute('Input.dispatchMouseEvent', [
                'type' => 'mouseWheel',
                'x' => 0,
                'y' => $coordinate->getY(),
                'deltaX' => 0,
                'deltaY' => $deltaPerStep,
            ]);

            usleep(random_int($this->delays['scroll']['min'], $this->delays['scroll']['max']) * 1000);
        }

        $this->scrollCoordinate = $coordinate;
    }

    public function getMouseCoordinate(): Coordinate
    {
        return $this->mouseCoordinate;
    }

    public function getScrollCoordinate(): Coordinate
    {
        return $this->scrollCoordinate;
    }
}

==> avodel-web-driver-5.6.0-a33968.zip: src/Mouse/Mouse.php <==
<?php

declare(strict_types=1);

namespace Avodel\WebDriver\Mouse;

use Facebook\WebDriver\Chrome\ChromeDevToolsDriver;
use Facebook\WebDriver\Remote\RemoteWebDriver;

class Mouse
{
    private Coordinate $mouseCoordinate;
    private Coordinate $scrollCoordinate;

    public function __construct(
        private readonly RemoteWebDriver $webDriver,
        private readonly MousePathStrategyInterface $mousePathStrategy,
        /**
         * @var array{scroll: array{min: int, max: int}, move: array{min: int, max: int}}
         */
        private readonly array $delays,
    ) {
        $this->mouseCoordinate = new Coordinate(random_int(600, 900), random_int(300, 600));
        $this->scrollCoordinate = new Coordinate(0, 0);
    }

    public function click(Coordinate $coordinate): void
    {
        $devTools = new ChromeDevToolsDriver($this->webDriver);

        $devTools->execute('Input.dispatchMouseEvent', [
            'type' => 'mousePressed',
            'x' => $coordinate->getX(),
            'y' => $coordinate->getY(),
            'button' => 'left',
            'clickCount' => 1,
        ]);
        $devTools->execute('Input.dispatchMouseEvent', [
            'type' => 'mouseReleased',
            'x' => $coordinate->getX(),
            'y' => $coordinate->getY(),
            'button' => 'left',
            'clickCount' => 1,
        ]);
    }

    /**
     * @return void
     */
    public function move(Coordinate $targetCoordinate): void
    {
        $mousePath = $this->mousePathStrategy->calculatePath(
            $this->mouseCoordinate,
            $targetCoordinate
        );

        $devTools = new ChromeDevToolsDriver($this->webDriver);

        foreach ($mousePath as $coordinate) {
            $devTools->execute('Input.dispatchMouseEvent', [
                'type' => 'mouseMoved',
                'x' => $coordinate->getX(),
                'y' => $coordinate->getY(),
            ]);

            usleep(random_int($this->delays['move']['min'], $this->delays['move']['max']) * 1000);

            $this->mouseCoordinate = $coordinate;
        }
    }

    public function scroll(Coordinate $coordinate): void
    {
        $scrollData = $this->webDriver->executeScript('return {
        scrollTop: window.scrollY,
        scrollHeight: document.documentElement.scrollHeight,
        windowHeight: window.innerHeight
    };');

        $currentScrollPosition = $scrollData['scrollTop'];
        $documentHeight = $scrollData['scrollHeight'];
        $windowHeight = $scrollData['windowHeight'];

        $targetY = $coordinate->getY();
        $deltaY = $targetY - $currentScrollPosition;

        if ($deltaY < 0 && $currentScrollPosition + $deltaY < 0) {
            $deltaY = -$currentScrollPosition;
        }

        if ($currentScrollPosition + $deltaY + $windowHeight >= $documentHeight) {
            $deltaY = $documentHeight - ($currentScrollPosition + $windowHeight);
        }

        if ($deltaY === 0) {
            return;
        }

        $deltaPerStep = ($deltaY < 50 && $deltaY > -50) ? $deltaY : (($deltaY < 0) ? -50 : 50);
        $steps = (int) ceil(abs($deltaY) / abs($deltaPerStep));

        $devTools = new ChromeDevToolsDriver($this->webDriver);

        for ($i = 0; $i < $steps; $i++) {
            $devTools->execute('Input.dispatchMouseEvent', [
                'type' => 'mouseWheel',
                'x' => 0,
                'y' => $coordinate->getY(),
                'deltaX' => 0,
                'deltaY' => $deltaPerStep,
            ]);

            usleep(random_int($this->delays['scroll']['min'], $this->delays['scroll']['max']) * 1000);
        }

        $this->scrollCoordinate = $coordinate;
    }

    public function getMouseCoordinate(): Coordinate
    {
        return $this->mouseCoordinate;
    }

    public function getScrollCoordinate(): Coordinate
    {
        return $this->scrollCoordinate;
    }
}
